@@ -330,19 +330,16 @@ shishi_apreq_from_file (Shishi * handle, Shishi_asn1 * apreq,
 int
 shishi_apreq_set_authenticator (Shishi * handle,
 				Shishi_asn1 apreq,
-				int etype, char *buf, int buflen)
+				int32_t etype, char *buf, int buflen)
 {
-  char format[BUFSIZ];
   int res;
 
-  res = shishi_asn1_write (handle, apreq, "authenticator.cipher",
-			   buf, buflen);
+  res =
+    shishi_asn1_write (handle, apreq, "authenticator.cipher", buf, buflen);
   if (res != SHISHI_OK)
     return res;
 
-  sprintf (format, "%d", etype);
-  res = shishi_asn1_write (handle, apreq, "authenticator.etype",
-			   format, 0);
+  res = shishi_asn1_write_int32 (handle, apreq, "authenticator.etype", etype);
   if (res != SHISHI_OK)
     return res;
 
@@ -368,8 +365,8 @@ shishi_apreq_add_authenticator (Shishi * handle,
 				int keyusage, Shishi_asn1 authenticator)
 {
   int res;
-  char buf[BUFSIZ];
-  int buflen;
+  char *buf;
+  size_t buflen;
   char *der;
   size_t derlen;
 
@@ -381,12 +378,13 @@ shishi_apreq_add_authenticator (Shishi * handle,
       return res;
     }
 
-  buflen = BUFSIZ;
-  res = shishi_encrypt (handle, key, keyusage, der, derlen, buf, &buflen);
-  free(der);
+  res = shishi_encrypt (handle, key, keyusage, der, derlen, &buf, &buflen);
+
+  free (der);
+
   if (res != SHISHI_OK)
     {
-      shishi_error_printf (handle, "des_encrypt fail\n");
+      shishi_error_printf (handle, "Cannot encrypt authenticator.\n");
       return res;
     }
 
@@ -411,7 +409,7 @@ shishi_apreq_set_ticket (Shishi * handle, Shishi_asn1 apreq,
 			 Shishi_asn1 ticket)
 {
   int res;
-  unsigned char format[BUFSIZ];
+  char *format;
   unsigned char buf[BUFSIZ];
   int buflen;
   int i, n;
@@ -421,8 +419,7 @@ shishi_apreq_set_ticket (Shishi * handle, Shishi_asn1 apreq,
   if (res != SHISHI_OK)
     return res;
 
-  res = shishi_asn1_write (handle, apreq, "ticket.tkt-vno",
-			   buf, buflen);
+  res = shishi_asn1_write (handle, apreq, "ticket.tkt-vno", buf, buflen);
   if (res != SHISHI_OK)
     return res;
 
@@ -436,8 +433,7 @@ shishi_apreq_set_ticket (Shishi * handle, Shishi_asn1 apreq,
     return res;
 
   buflen = BUFSIZ;
-  res = shishi_asn1_read (handle, ticket, "sname.name-type",
-			  buf, &buflen);
+  res = shishi_asn1_read (handle, ticket, "sname.name-type", buf, &buflen);
   if (res != SHISHI_OK)
     return res;
 
@@ -458,21 +454,22 @@ shishi_apreq_set_ticket (Shishi * handle, Shishi_asn1 apreq,
       if (res != SHISHI_OK)
 	return res;
 
-      sprintf (format, "sname.name-string.?%d", i);
       buflen = BUFSIZ;
+      asprintf (&format, "sname.name-string.?%d", i);
       res = shishi_asn1_read (handle, ticket, format, buf, &buflen);
+      free (format);
       if (res != SHISHI_OK)
 	return res;
 
-      sprintf (format, "ticket.sname.name-string.?%d", i);
+      asprintf (&format, "ticket.sname.name-string.?%d", i);
       res = shishi_asn1_write (handle, apreq, format, buf, buflen);
+      free (format);
       if (res != SHISHI_OK)
 	return res;
     }
 
   buflen = BUFSIZ;
-  res = shishi_asn1_read (handle, ticket, "enc-part.etype",
-			  buf, &buflen);
+  res = shishi_asn1_read (handle, ticket, "enc-part.etype", buf, &buflen);
   if (res != SHISHI_OK)
     return res;
 
@@ -482,14 +479,12 @@ shishi_apreq_set_ticket (Shishi * handle, Shishi_asn1 apreq,
     return res;
 
   buflen = BUFSIZ;
-  res = shishi_asn1_read (handle, ticket, "enc-part.kvno",
-			  buf, &buflen);
+  res = shishi_asn1_read (handle, ticket, "enc-part.kvno", buf, &buflen);
   if (res != SHISHI_OK && res != SHISHI_ASN1_NO_ELEMENT)
     return res;
 
   if (res == SHISHI_ASN1_NO_ELEMENT)
-    res = shishi_asn1_write (handle, apreq, "ticket.enc-part.kvno",
-			     NULL, 0);
+    res = shishi_asn1_write (handle, apreq, "ticket.enc-part.kvno", NULL, 0);
   else
     res = shishi_asn1_write (handle, apreq, "ticket.enc-part.kvno",
 			     buf, buflen);
@@ -497,8 +492,7 @@ shishi_apreq_set_ticket (Shishi * handle, Shishi_asn1 apreq,
     return res;
 
   buflen = BUFSIZ;
-  res = shishi_asn1_read (handle, ticket, "enc-part.cipher",
-			  buf, &buflen);
+  res = shishi_asn1_read (handle, ticket, "enc-part.cipher", buf, &buflen);
   if (res != SHISHI_OK)
     return res;
 
@@ -513,12 +507,7 @@ shishi_apreq_set_ticket (Shishi * handle, Shishi_asn1 apreq,
 int
 shishi_apreq_options (Shishi * handle, Shishi_asn1 apreq, int *flags)
 {
-  int len = sizeof (*flags);
-  int res;
-  *flags = 0;
-  res = shishi_asn1_field (handle, apreq, (char *) flags, &len,
-			   "ap-options");
-  return res;
+  return shishi_asn1_read_bitstring (handle, apreq, "ap-options", flags);
 }
 
 int
@@ -546,9 +535,7 @@ shishi_apreq_options_set (Shishi * handle, Shishi_asn1 apreq, int options)
 {
   int res;
 
-  res = shishi_asn1_write (handle, apreq, "ap-options",
-			   (char *) &options,
-			   SHISHI_APREQ_DEFAULT_AP_OPTIONS_LEN);
+  res = shishi_asn1_write_bitstring (handle, apreq, "ap-options", options);
   if (res != SHISHI_OK)
     return res;
 
@@ -586,10 +573,9 @@ shishi_apreq_options_add (Shishi * handle, Shishi_asn1 apreq, int option)
  **/
 int
 shishi_apreq_get_authenticator_etype (Shishi * handle,
-				      Shishi_asn1 apreq, int *etype)
+				      Shishi_asn1 apreq, int32_t * etype)
 {
-  return shishi_asn1_integer_field (handle, apreq, etype,
-				    "authenticator.etype");
+  return shishi_asn1_read_int32 (handle, apreq, "authenticator.etype", etype);
 }
 
 /**
@@ -607,7 +593,7 @@ shishi_apreq_get_ticket (Shishi * handle,
 			 Shishi_asn1 apreq, Shishi_asn1 * ticket)
 {
   unsigned char buf[BUFSIZ];
-  unsigned char format[BUFSIZ];
+  char *format;
   int buflen;
   int res;
   int i, n;
@@ -619,8 +605,7 @@ shishi_apreq_get_ticket (Shishi * handle,
     return SHISHI_ASN1_ERROR;
 
   buflen = BUFSIZ;
-  res = shishi_asn1_read (handle, apreq, "ticket.tkt-vno",
-			  buf, &buflen);
+  res = shishi_asn1_read (handle, apreq, "ticket.tkt-vno", buf, &buflen);
   if (res != SHISHI_OK)
     goto error;
 
@@ -643,14 +628,12 @@ shishi_apreq_get_ticket (Shishi * handle,
   if (res != SHISHI_OK)
     goto error;
 
-  res = shishi_asn1_write (handle, *ticket, "sname.name-type",
-			   buf, buflen);
+  res = shishi_asn1_write (handle, *ticket, "sname.name-type", buf, buflen);
   if (res != SHISHI_OK)
     goto error;
 
   res = shishi_asn1_number_of_elements (handle, apreq,
-					"ticket.sname.name-string",
-					&n);
+					"ticket.sname.name-string", &n);
   if (res != SHISHI_OK)
     goto error;
 
@@ -661,14 +644,16 @@ shishi_apreq_get_ticket (Shishi * handle,
       if (res != SHISHI_OK)
 	goto error;
 
-      sprintf (format, "ticket.sname.name-string.?%d", i);
       buflen = BUFSIZ;
+      asprintf (&format, "ticket.sname.name-string.?%d", i);
       res = shishi_asn1_read (handle, apreq, format, buf, &buflen);
+      free (format);
       if (res != SHISHI_OK)
 	goto error;
 
-      sprintf (format, "sname.name-string.?%d", i);
+      asprintf (&format, "sname.name-string.?%d", i);
       res = shishi_asn1_write (handle, *ticket, format, buf, buflen);
+      free (format);
       if (res != SHISHI_OK)
 	goto error;
     }
@@ -679,8 +664,7 @@ shishi_apreq_get_ticket (Shishi * handle,
   if (res != SHISHI_OK)
     goto error;
 
-  res = shishi_asn1_write (handle, *ticket, "enc-part.etype",
-			   buf, buflen);
+  res = shishi_asn1_write (handle, *ticket, "enc-part.etype", buf, buflen);
   if (res != SHISHI_OK)
     goto error;
 
@@ -691,11 +675,9 @@ shishi_apreq_get_ticket (Shishi * handle,
     goto error;
 
   if (res == SHISHI_ASN1_NO_ELEMENT)
-    res = shishi_asn1_write (handle, *ticket, "enc-part.kvno",
-			     NULL, 0);
+    res = shishi_asn1_write (handle, *ticket, "enc-part.kvno", NULL, 0);
   else
-    res = shishi_asn1_write (handle, *ticket, "enc-part.kvno",
-			     buf, buflen);
+    res = shishi_asn1_write (handle, *ticket, "enc-part.kvno", buf, buflen);
   if (res != SHISHI_OK)
     goto error;
 
@@ -705,8 +687,7 @@ shishi_apreq_get_ticket (Shishi * handle,
   if (res != SHISHI_OK)
     goto error;
 
-  res = shishi_asn1_write (handle, *ticket, "enc-part.cipher",
-			   buf, buflen);
+  res = shishi_asn1_write (handle, *ticket, "enc-part.cipher", buf, buflen);
   if (res != SHISHI_OK)
     goto error;
 
@@ -725,8 +706,8 @@ shishi_apreq_decrypt (Shishi * handle,
 {
   int res;
   int i;
-  size_t buflen = BUFSIZ;
-  char buf[BUFSIZ];
+  char *buf;
+  size_t buflen;
   char cipher[BUFSIZ];
   int cipherlen;
   int etype;
@@ -745,12 +726,10 @@ shishi_apreq_decrypt (Shishi * handle,
     return res;
 
   res = shishi_decrypt (handle, key, keyusage,
-			cipher, cipherlen, buf, &buflen);
+			cipher, cipherlen, &buf, &buflen);
 
   if (res != SHISHI_OK)
     {
-      if (VERBOSE (handle))
-	printf ("decrypt failed: %s\n", shishi_strerror_details (handle));
       shishi_error_printf (handle,
 			   "decrypt fail, most likely wrong password\n");
       return SHISHI_APREQ_DECRYPT_FAILED;
