@@ -110,6 +110,26 @@ shishi_as_req (Shishi_as * as)
   return as->asreq;
 }
 
+/**
+ * shishi_as_req_build:
+ * @as: structure that holds information about AS exchange
+ *
+ * Possibly remove unset fields (e.g., rtime).
+ *
+ * Return value: Returns SHISHI_OK iff successful.
+ **/
+int
+shishi_as_req_build (Shishi_as * as)
+{
+  int res;
+
+  res = shishi_kdcreq_build (as->handle, as->asreq);
+  if (res != SHISHI_OK)
+      return res;
+
+  return SHISHI_OK;
+}
+
 /**
  * shishi_as_req_set:
  * @as: structure that holds information about AS exchange
