@@ -215,14 +215,14 @@ shishi_asn1_write_bitstring (Shishi * handle, Shishi_asn1 node,
 
   for (i = 0; i < 4; i++)
     {
-      buf[i] |= ((((flags & (0xFF << 8 * i)) >> 7) & 0x01) |
-		 (((flags & (0xFF << 8 * i)) >> 5) & 0x02) |
-		 (((flags & (0xFF << 8 * i)) >> 3) & 0x04) |
-		 (((flags & (0xFF << 8 * i)) >> 1) & 0x08) |
-		 (((flags & (0xFF << 8 * i)) << 1) & 0x10) |
-		 (((flags & (0xFF << 8 * i)) << 3) & 0x20) |
-		 (((flags & (0xFF << 8 * i)) << 5) & 0x40) |
-		 (((flags & (0xFF << 8 * i)) << 7) & 0x80)) << (8 * i);
+      buf[i] = ((((flags & (0xFF << 8 * i)) >> 7) & 0x01) |
+		(((flags & (0xFF << 8 * i)) >> 5) & 0x02) |
+		(((flags & (0xFF << 8 * i)) >> 3) & 0x04) |
+		(((flags & (0xFF << 8 * i)) >> 1) & 0x08) |
+		(((flags & (0xFF << 8 * i)) << 1) & 0x10) |
+		(((flags & (0xFF << 8 * i)) << 3) & 0x20) |
+		(((flags & (0xFF << 8 * i)) << 5) & 0x40) |
+		(((flags & (0xFF << 8 * i)) << 7) & 0x80)) << (8 * i);
     }
 
   buflen = sizeof (buf);
@@ -288,14 +288,14 @@ int
 shishi_asn1_read_uint32 (Shishi * handle, Shishi_asn1 node,
 			 const char *field, uint32_t * i)
 {
-  shishi_asn1_read_int32 (handle, node, field, (int32_t *) i);
+  return shishi_asn1_read_int32 (handle, node, field, (int32_t *) i);
 }
 
 int
 shishi_asn1_read_integer (Shishi * handle, Shishi_asn1 node,
 			  const char *field, int *i)
 {
-  shishi_asn1_read_int32 (handle, node, field, (int32_t *) i);
+  return shishi_asn1_read_int32 (handle, node, field, (int32_t *) i);
 }
 
 int
@@ -384,6 +384,26 @@ shishi_asn1_optional_field (Shishi * handle,
   return SHISHI_OK;
 }
 
+int
+shishi_asn1_empty_field (Shishi * handle,
+			 Shishi_asn1 node,
+			 char *data, size_t * datalen, const char *field)
+{
+  int rc;
+
+  rc = asn1_read_value (node, field, (unsigned char *) data, (int *) datalen);
+  if (rc != ASN1_SUCCESS && rc != ASN1_VALUE_NOT_FOUND)
+    {
+      shishi_error_set (handle, libtasn1_strerror (rc));
+      return SHISHI_ASN1_ERROR;
+    }
+
+  if (rc == ASN1_VALUE_NOT_FOUND)
+    *datalen = 0;
+
+  return SHISHI_OK;
+}
+
 #define SHISHI_TICKET_DEFAULT_TKTVNO "5"
 #define SHISHI_TICKET_DEFAULT_TKTVNO_LEN 0
 
