@@ -33,8 +33,7 @@ shishi_enckdcreppart (Shishi * handle)
     return NULL;
 
   /* XXX remove these two: */
-  res = shishi_asn1_write (handle, node, "key-expiration",
-			   NULL, 0);
+  res = shishi_asn1_write (handle, node, "key-expiration", NULL, 0);
   if (res != SHISHI_OK)
     puts ("urk");
   res = shishi_asn1_write (handle, node, "caddr", NULL, 0);
@@ -55,8 +54,7 @@ shishi_encasreppart (Shishi * handle)
     return NULL;
 
   /* XXX remove these two: */
-  res = shishi_asn1_write (handle, node, "key-expiration",
-			   NULL, 0);
+  res = shishi_asn1_write (handle, node, "key-expiration", NULL, 0);
   if (res != SHISHI_OK)
     puts ("urk");
   res = shishi_asn1_write (handle, node, "caddr", NULL, 0);
@@ -87,10 +85,10 @@ shishi_enckdcreppart_get_key (Shishi * handle,
   int res;
   char buf[BUFSIZ];
   int buflen;
-  int keytype;
+  int32_t keytype;
 
-  res = shishi_asn1_integer_field (handle, enckdcreppart, &keytype,
-				   "key.keytype");
+  res = shishi_asn1_read_int32 (handle, enckdcreppart,
+				"key.keytype", &keytype);
   if (res != SHISHI_OK)
     return res;
 
@@ -123,18 +121,13 @@ shishi_enckdcreppart_key_set (Shishi * handle,
 			      Shishi_asn1 enckdcreppart, Shishi_key * key)
 {
   int res;
-  char buf[BUFSIZ];
-  int keytype;
 
-  keytype = shishi_key_type (key);
-  sprintf (buf, "%d", keytype);
-  res = shishi_asn1_write (handle, enckdcreppart, "key.keytype",
-			   buf, 0);
+  res = shishi_asn1_write_integer (handle, enckdcreppart, "key.keytype",
+				   shishi_key_type (key));
   if (res != SHISHI_OK)
     return SHISHI_ASN1_ERROR;
 
-  res = shishi_asn1_write (handle, enckdcreppart,
-			   "key.keyvalue",
+  res = shishi_asn1_write (handle, enckdcreppart, "key.keyvalue",
 			   shishi_key_value (key), shishi_key_length (key));
   if (res != SHISHI_OK)
     return SHISHI_ASN1_ERROR;
@@ -154,16 +147,11 @@ shishi_enckdcreppart_key_set (Shishi * handle,
  **/
 int
 shishi_enckdcreppart_nonce_set (Shishi * handle,
-				Shishi_asn1 enckdcreppart,
-				unsigned long nonce)
+				Shishi_asn1 enckdcreppart, uint32_t nonce)
 {
   int res;
-  char *format;
 
-  shishi_asprintf (&format, "%ld", nonce);
-  res = shishi_asn1_write (handle, enckdcreppart, "nonce",
-			   format, 0);
-  free (format);
+  res = shishi_asn1_write_integer (handle, enckdcreppart, "nonce", nonce);
   if (res != SHISHI_OK)
     return res;
 
@@ -185,11 +173,8 @@ shishi_enckdcreppart_flags_set (Shishi * handle,
 				Shishi_asn1 enckdcreppart, int flags)
 {
   int res;
-  char buf[BUFSIZ];
 
-  sprintf (buf, "%d", flags);
-  res = shishi_asn1_write (handle, enckdcreppart, "flags",
-			   buf, 0);
+  res = shishi_asn1_write_integer (handle, enckdcreppart, "flags", flags);
   if (res != SHISHI_OK)
     return SHISHI_ASN1_ERROR;
 
@@ -218,62 +203,51 @@ shishi_enckdcreppart_populate_encticketpart (Shishi * handle,
   int res;
 
   buflen = BUFSIZ;
-  res = shishi_asn1_read (handle, encticketpart, "flags",
-			  buf, &buflen);
+  res = shishi_asn1_read (handle, encticketpart, "flags", buf, &buflen);
   if (res != SHISHI_OK)
     return SHISHI_ASN1_ERROR;
 
-  res = shishi_asn1_write (handle, enckdcreppart, "flags",
-			   buf, buflen);
+  res = shishi_asn1_write (handle, enckdcreppart, "flags", buf, buflen);
   if (res != SHISHI_OK)
     return SHISHI_ASN1_ERROR;
 
   buflen = BUFSIZ;
-  res = shishi_asn1_read (handle, encticketpart, "authtime",
-			  buf, &buflen);
+  res = shishi_asn1_read (handle, encticketpart, "authtime", buf, &buflen);
   if (res != SHISHI_OK)
     return SHISHI_ASN1_ERROR;
 
-  res = shishi_asn1_write (handle, enckdcreppart, "authtime",
-			   buf, buflen);
+  res = shishi_asn1_write (handle, enckdcreppart, "authtime", buf, buflen);
   if (res != SHISHI_OK)
     return SHISHI_ASN1_ERROR;
 
   buflen = BUFSIZ;
-  res = shishi_asn1_read (handle, encticketpart, "starttime",
-			  buf, &buflen);
+  res = shishi_asn1_read (handle, encticketpart, "starttime", buf, &buflen);
   if (res != SHISHI_OK && res != SHISHI_ASN1_NO_ELEMENT)
     return SHISHI_ASN1_ERROR;
 
   if (res == SHISHI_ASN1_NO_ELEMENT)
-    res = shishi_asn1_write (handle, enckdcreppart, "starttime",
-			     NULL, 0);
+    res = shishi_asn1_write (handle, enckdcreppart, "starttime", NULL, 0);
   else
-    res = shishi_asn1_write (handle, enckdcreppart, "starttime",
-			     buf, buflen);
+    res = shishi_asn1_write (handle, enckdcreppart, "starttime", buf, buflen);
   if (res != SHISHI_OK)
     return SHISHI_ASN1_ERROR;
 
   buflen = BUFSIZ;
-  res = shishi_asn1_read (handle, encticketpart, "endtime",
-			  buf, &buflen);
+  res = shishi_asn1_read (handle, encticketpart, "endtime", buf, &buflen);
   if (res != SHISHI_OK)
     return SHISHI_ASN1_ERROR;
 
-  res = shishi_asn1_write (handle, enckdcreppart, "endtime",
-			   buf, buflen);
+  res = shishi_asn1_write (handle, enckdcreppart, "endtime", buf, buflen);
   if (res != SHISHI_OK)
     return SHISHI_ASN1_ERROR;
 
   buflen = BUFSIZ;
-  res = shishi_asn1_read (handle, encticketpart, "renew-till",
-			  buf, &buflen);
+  res = shishi_asn1_read (handle, encticketpart, "renew-till", buf, &buflen);
   if (res != SHISHI_OK && res != SHISHI_ASN1_NO_ELEMENT)
     return SHISHI_ASN1_ERROR;
 
   if (res == SHISHI_ASN1_NO_ELEMENT)
-    res = shishi_asn1_write (handle, enckdcreppart,
-			     "renew-till", NULL, 0);
+    res = shishi_asn1_write (handle, enckdcreppart, "renew-till", NULL, 0);
   else
     res = shishi_asn1_write (handle, enckdcreppart,
 			     "renew-till", buf, buflen);
@@ -302,8 +276,7 @@ shishi_enckdcreppart_srealm_set (Shishi * handle,
 {
   int res = SHISHI_OK;
 
-  res = shishi_asn1_write (handle, enckdcreppart,
-			   "srealm", srealm, 0);
+  res = shishi_asn1_write (handle, enckdcreppart, "srealm", srealm, 0);
   if (res != SHISHI_OK)
     return res;
 
@@ -329,13 +302,11 @@ shishi_enckdcreppart_sname_set (Shishi * handle,
 				Shishi_name_type name_type, char *sname[])
 {
   int res = SHISHI_OK;
-  char buf[BUFSIZ];
   int i;
+  char *buf;
 
-  sprintf (buf, "%d", name_type);
-
-  res = shishi_asn1_write (handle, enckdcreppart,
-			   "sname.name-type", buf, 0);
+  res = shishi_asn1_write_integer (handle, enckdcreppart,
+				   "sname.name-type", name_type);
   if (res != SHISHI_OK)
     return res;
 
@@ -347,13 +318,14 @@ shishi_enckdcreppart_sname_set (Shishi * handle,
   i = 1;
   while (sname[i - 1])
     {
-      res = shishi_asn1_write (handle, enckdcreppart,
-			       "sname.name-string", "NEW", 1);
+      res = shishi_asn1_write (handle, enckdcreppart, "sname.name-string",
+			       "NEW", 1);
       if (res != SHISHI_OK)
 	return res;
 
-      sprintf (buf, "sname.name-string.?%d", i);
+      asprintf (&buf, "sname.name-string.?%d", i);
       res = shishi_asn1_write (handle, enckdcreppart, buf, sname[i - 1], 0);
+      free (buf);
       if (res != SHISHI_OK)
 	return res;
 
