@@ -98,17 +98,10 @@ _shishi_kdcreq (Shishi * handle, int as)
   if (res != SHISHI_OK)
     goto error;
 
-  res = shishi_asn1_write (handle, node, "req-body.from", NULL, 0);
-  if (res != SHISHI_OK)
-    goto error;
-
   res = shishi_asn1_write (handle, node, "req-body.till",
 			   shishi_generalize_time (handle,
-						   time (NULL) + 1000), 0);
-  if (res != SHISHI_OK)
-    goto error;
-
-  res = shishi_asn1_write (handle, node, "req-body.rtime", NULL, 0);
+						   time (NULL) +
+						   handle->ticketlife), 0);
   if (res != SHISHI_OK)
     goto error;
 
@@ -565,6 +558,55 @@ shishi_kdcreq_set_etype (Shishi * handle,
   return SHISHI_OK;
 }
 
+int
+shishi_kdcreq_options (Shishi * handle, Shishi_asn1 kdcreq, int *flags)
+{
+  return shishi_asn1_read_bitstring (handle, kdcreq,
+				     "req-body.kdc-options", flags);
+}
+
+int
+shishi_kdcreq_renewable_p (Shishi * handle, Shishi_asn1 kdcreq)
+{
+  int options = 0;
+
+  shishi_kdcreq_options (handle, kdcreq, &options);
+
+  return options & SHISHI_KDCOPTIONS_RENEWABLE;
+}
+
+int
+shishi_kdcreq_options_set (Shishi * handle, Shishi_asn1 kdcreq, int options)
+{
+  int res;
+
+  res = shishi_asn1_write_bitstring (handle, kdcreq,
+				     "req-body.kdc-options", options);
+  if (res != SHISHI_OK)
+    return res;
+
+  return SHISHI_OK;
+}
+
+int
+shishi_kdcreq_options_add (Shishi * handle, Shishi_asn1 kdcreq, int option)
+{
+  int options;
+  int res;
+
+  res = shishi_kdcreq_options (handle, kdcreq, &options);
+  if (res != SHISHI_OK)
+    return res;
+
+  options |= option;
+
+  res = shishi_kdcreq_options_set (handle, kdcreq, options);
+  if (res != SHISHI_OK)
+    return res;
+
+  return SHISHI_OK;
+}
+
 /**
  * shishi_kdcreq_clear_padata:
  * @handle: shishi handle as allocated by shishi_init().
@@ -671,3 +713,54 @@ shishi_kdcreq_add_padata_tgs (Shishi * handle,
 
   return res;
 }
+
+int
+shishi_kdcreq_build (Shishi * handle, Shishi_asn1 kdcreq)
+{
+  char buffer[BUFSIZ];		/* XXX dynamically allocate this */
+  int buflen;
+  int res;
+
+  if (VERBOSE (handle))
+    printf ("Building KDC-REQ...\n");
+
+  buflen = sizeof(buffer);
+  res = shishi_asn1_empty_field (handle, kdcreq, buffer, &buflen,
+				 "req-body.rtime");
+  if (res != SHISHI_OK)
+    {
+      shishi_error_printf (handle, "Could not read rtime\n");
+      return res;
+    }
+
+  if (buflen == 0)
+    {
+      res = shishi_asn1_write (handle, kdcreq, "req-body.rtime", NULL, 0);
+      if (res != SHISHI_OK)
+	{
+	  shishi_error_printf (handle, "Could not write rtime\n");
+	  return res;
+	}
+    }
+
+  buflen = sizeof(buffer);
+  res = shishi_asn1_empty_field (handle, kdcreq, buffer, &buflen,
+				 "req-body.from");
+  if (res != SHISHI_OK)
+    {
+      shishi_error_printf (handle, "Could not read from\n");
+      return res;
+    }
+
+  if (buflen == 0)
+    {
+      res = shishi_asn1_write (handle, kdcreq, "req-body.from", NULL, 0);
+      if (res != SHISHI_OK)
+	{
+	  shishi_error_printf (handle, "Could not write from\n");
+	  return res;
+	}
+    }
+
+  return SHISHI_OK;
+}
