@@ -208,7 +208,7 @@ shishi_kdc_sendrecv_srv (Shishi * handle, char *realm,
 				    outdata, outlen, rrs);
   else
     {
-      shishi_error_printf (handle, "No KDC SRV RRs for realm %s\n", realm);
+      shishi_error_printf (handle, "No KDC SRV RRs for realm %s", realm);
       rc = SHISHI_KDC_NOT_KNOWN_FOR_REALM;
     }
 
@@ -217,6 +217,51 @@ shishi_kdc_sendrecv_srv (Shishi * handle, char *realm,
   return rc;
 }
 
+static int
+shishi_kdc_sendrecv_direct (Shishi * handle, char *realm,
+			    const char *indata, size_t inlen,
+			    char *outdata, size_t * outlen)
+{
+  struct servent *se;
+  struct addrinfo hints;
+  struct addrinfo *ai;
+  char *port;
+  int rc;
+
+  if (VERBOSE (handle))
+    printf ("Trying direct realm host mapping for %s...\n", realm);
+
+  se = getservbyname ("kerberos", NULL);
+  if (se)
+    asprintf (&port, "%d", ntohs(se->s_port));
+  else
+    asprintf (&port, "%d", 88);
+
+  memset (&hints, 0, sizeof(hints));
+  hints.ai_socktype = SOCK_DGRAM;
+  rc = getaddrinfo (realm, port, &hints, &ai);
+
+  free (port);
+
+  if (rc != 0)
+    {
+      shishi_error_printf (handle, "No direct realm host for realm %s", realm);
+      return SHISHI_KDC_NOT_KNOWN_FOR_REALM;
+    }
+
+  if (VERBOSE (handle))
+    printf ("Sending to %s:%s (%s)...\n", realm, port,
+	    inet_ntoa (((struct sockaddr_in *)ai->ai_addr)->sin_addr));
+
+  rc = shishi_sendrecv_udp (handle, ai->ai_addr,
+			    indata, inlen, outdata, outlen,
+			    handle->kdctimeout);
+
+  freeaddrinfo (ai);
+
+  return rc;
+}
+
 int
 shishi_kdc_sendrecv (Shishi * handle, char *realm,
 		     const char *indata, size_t inlen,
@@ -230,6 +275,9 @@ shishi_kdc_sendrecv (Shishi * handle, char *realm,
   if (rc == SHISHI_KDC_TIMEOUT || rc == SHISHI_KDC_NOT_KNOWN_FOR_REALM)
     rc = shishi_kdc_sendrecv_srv (handle, realm,
 				  indata, inlen, outdata, outlen);
+  if (rc == SHISHI_KDC_TIMEOUT || rc == SHISHI_KDC_NOT_KNOWN_FOR_REALM)
+    rc = shishi_kdc_sendrecv_direct (handle, realm,
+				     indata, inlen, outdata, outlen);
 
   return rc;
 }
