@@ -88,6 +88,51 @@ shishi_principal_default_set (Shishi * handle, const char *principal)
     handle->default_principal = NULL;
 }
 
+int
+shishi_parse_name (Shishi * handle, const char *name,
+		   char **principal, char **realm)
+{
+  char *p = (char *) name;
+  char *q;
+  int escaped = 0;
+
+  if (!name)
+    return SHISHI_INVALID_PRINCIPAL_NAME;
+
+  while (*p && (*p != '@' || escaped))
+    if (escaped)
+      escaped = 0;
+    else if (*p++ == '\\')
+      escaped = 1;
+
+  if (principal)
+    {
+      *principal = xstrndup (name, p - name + 1);
+      (*principal)[p - name] = '\0';
+    }
+
+  if (*p)
+    {
+      q = ++p;
+
+      while (*q)
+	if (escaped)
+	  escaped = 0;
+	else if (*q++ == '\\')
+	  escaped = 1;
+
+      if (escaped)
+	return SHISHI_INVALID_PRINCIPAL_NAME;
+
+      if (realm)
+	*realm = xstrdup (p);
+    }
+  else if (realm)
+    *realm = strdup(shishi_realm_default (handle));
+
+  return SHISHI_OK;
+}
+
 /*
   2.1.1. Kerberos Principal Name Form
 
@@ -348,3 +393,22 @@ shishi_principal_set (Shishi * handle,
 
   return SHISHI_OK;
 }
+
+char *
+shishi_server_for_local_service (Shishi * handle, const char *service)
+{
+  char buf[HOST_NAME_MAX];
+  int ret;
+
+  strcpy (buf, service);
+  strcat (buf, "/");
+
+  ret = gethostname (&buf[strlen (service) + 1],
+		     sizeof (buf) - strlen (service) - 1);
+  buf[sizeof (buf) - 1] = '\0';
+
+  if (ret != 0)
+    strcpy (&buf[strlen (service) + 1], "localhost");
+
+  return xstrdup(buf);
+}
