@@ -27,7 +27,7 @@
 #include <time.h>		/* time_t */
 #include <shishi-int.h>		/* uint32_t */
 
-#define SHISHI_VERSION "0.0.1"
+#define SHISHI_VERSION "0.0.2"
 
 /* Error codes */
 enum
@@ -74,7 +74,9 @@ enum
   SHISHI_PKCS5_INVALID_PRF = 39,
   SHISHI_PKCS5_INVALID_ITERATION_COUNT = 40,
   SHISHI_PKCS5_INVALID_DERIVED_KEY_LENGTH = 41,
-  SHISHI_PKCS5_DERIVED_KEY_TOO_LONG = 42
+  SHISHI_PKCS5_DERIVED_KEY_TOO_LONG = 42,
+  SHISHI_INVALID_PRINCIPAL_NAME = 43,
+  SHISHI_INVALID_ARGUMENT = 44
 };
 
 /* This is not specified in the ASN.1 syntax for some reason. */
@@ -160,12 +162,12 @@ Shishi_KDCOptions;
 
 typedef enum
 {
-  SHISHI_MSGTYPE_AS_REQ = 10,
-  SHISHI_MSGTYPE_AS_REP = 11,
-  SHISHI_MSGTYPE_TGS_REQ = 12,
-  SHISHI_MSGTYPE_TGS_REP = 13
+  SHISHI_MSGTYPE_AS_REQ = 0x10,
+  SHISHI_MSGTYPE_AS_REP = 0x11,
+  SHISHI_MSGTYPE_TGS_REQ = 0x12,
+  SHISHI_MSGTYPE_TGS_REP = 0x13
 }
-Shihi_msgtype;
+Shishi_msgtype;
 
 typedef enum
 {
@@ -175,7 +177,7 @@ typedef enum
   SHISHI_LRTYPE_LAST_RENEWAL = 4,
   SHISHI_LRTYPE_LAST_REQUEST = 5
 }
-Shihi_lrtype;
+Shishi_lrtype;
 
 typedef enum
 {
@@ -298,6 +300,10 @@ struct Shishi_tkts_hint
   int flags;
   int32_t etype;
   char *passwd;
+  time_t starttime;
+  time_t endtime;
+  time_t renew_till;
+  int renewable;
 };
 typedef struct Shishi_tkts_hint Shishi_tkts_hint;
 
@@ -379,6 +385,10 @@ extern int shishi_principal_name_set (Shishi * handle,
 extern int shishi_principal_set (Shishi * handle,
 				 Shishi_asn1 namenode,
 				 const char *namefield, const char *name);
+extern int shishi_parse_name (Shishi * handle, const char *name,
+			      char **principal, char **realm);
+extern char *shishi_server_for_local_service (Shishi * handle,
+					      const char *service);
 
 /* ticket.c */
 extern int shishi_ticket_sname_get (Shishi * handle,
@@ -446,7 +456,7 @@ extern int shishi_tkt_serverrealm_set (Shishi_tkt * tkt,
 extern int shishi_tkt_build (Shishi_tkt * tkt, Shishi_key * key);
 extern int shishi_tkt_lastreq (Shishi_tkt * tkt,
 			       char *lrtime, int *lrtimelen, int lrtype);
-extern time_t shishi_tkt_lastreqc (Shishi_tkt * tkt, Shihi_lrtype lrtype);
+extern time_t shishi_tkt_lastreqc (Shishi_tkt * tkt, Shishi_lrtype lrtype);
 extern int shishi_tkt_lastreq_pretty_print (Shishi_tkt * tkt, FILE * fh);
 extern int shishi_tkt_authtime (Shishi_tkt * tkt,
 				char *authtime, int *authtimelen);
@@ -634,6 +644,7 @@ extern int shishi_authenticator_save (Shishi * handle, FILE * fh,
 
 /* as.c */
 extern Shishi_asn1 shishi_as_req (Shishi_as * as);
+extern int shishi_as_req_build (Shishi_as * as);
 extern void shishi_as_req_set (Shishi_as * as, Shishi_asn1 asreq);
 extern int shishi_as_req_der (Shishi_as * as, char *out, int *outlen);
 extern int shishi_as_req_der_set (Shishi_as * as, char *der, size_t derlen);
@@ -740,6 +751,13 @@ extern int shishi_kdcreq_etype (Shishi * handle, Shishi_asn1 kdcreq,
 				int32_t * etype, int netype);
 extern int shishi_kdcreq_set_etype (Shishi * handle, Shishi_asn1 kdcreq,
 				    int32_t * etype, int netype);
+extern int shishi_kdcreq_options (Shishi * handle, Shishi_asn1 kdcreq,
+				  int *flags);
+extern int shishi_kdcreq_renewable_p (Shishi * handle, Shishi_asn1 kdcreq);
+extern int shishi_kdcreq_options_set (Shishi * handle, Shishi_asn1 kdcreq,
+				      int options);
+extern int shishi_kdcreq_options_add (Shishi * handle, Shishi_asn1 kdcreq,
+				      int option);
 extern int shishi_kdcreq_clear_padata (Shishi * handle, Shishi_asn1 kdcreq);
 extern int shishi_kdcreq_add_padata (Shishi * handle,
 				     Shishi_asn1 kdcreq,
@@ -747,6 +765,7 @@ extern int shishi_kdcreq_add_padata (Shishi * handle,
 extern int shishi_kdcreq_add_padata_tgs (Shishi * handle,
 					 Shishi_asn1 kdcreq,
 					 Shishi_asn1 apreq);
+extern int shishi_kdcreq_build (Shishi * handle, Shishi_asn1 kdcreq);
 
 /* enckdcreppart.c */
 extern Shishi_asn1 shishi_enckdcreppart (Shishi * handle);
@@ -1012,6 +1031,10 @@ extern int shishi_asn1_optional_field (Shishi * handle,
 				       Shishi_asn1 node,
 				       char *data, size_t * datalen,
 				       const char *field);
+extern int shishi_asn1_empty_field (Shishi * handle,
+				    Shishi_asn1 node,
+				    char *data, size_t * datalen,
+				    const char *field);
 extern int shishi_asn1_number_of_elements (Shishi * handle,
 					   Shishi_asn1 node,
 					   const char *field, int *n);
@@ -1188,20 +1211,26 @@ extern int shishi_key_from_string (Shishi * handle,
 				   const char *parameter,
 				   Shishi_key ** outkey);
 
+/* keys.c */
+extern Shishi_key *
+shishi_keys_for_serverrealm_in_file (Shishi * handle,
+				     const char *filename,
+				     const char *server,
+				     const char *realm);
+extern Shishi_key *
+shishi_keys_for_server_in_file (Shishi * handle,
+				const char *filename,
+				const char *server);
+extern Shishi_key *
+shishi_keys_for_localservicerealm_in_file (Shishi * handle,
+					   const char *filename,
+					   const char *service,
+					   const char *realm);
+
 /* hostkeys.c */
 extern const char *shishi_hostkeys_default_file (Shishi * handle);
 extern void shishi_hostkeys_default_file_set (Shishi * handle,
 					      const char *hostkeysfile);
-extern Shishi_key *shishi_hostkeys_for_serverrealm_in_file (Shishi * handle,
-							    const char
-							    *filename,
-							    const char
-							    *server,
-							    const char
-							    *realm);
-extern Shishi_key *shishi_hostkeys_for_server_in_file (Shishi * handle,
-						       const char *filename,
-						       const char *server);
 extern Shishi_key *shishi_hostkeys_for_server (Shishi * handle,
 					       const char *server);
 extern Shishi_key *shishi_hostkeys_for_serverrealm (Shishi * handle,
