@@ -154,14 +154,20 @@ shishi_tgs_req (Shishi_tgs * tgs)
 int
 shishi_tgs_req_build (Shishi_tgs * tgs)
 {
+  char rtime[BUFSIZ];		/* XXX dynamically allocate this */
+  int rtimelen;
   int res;
   int apoptions;
 
   if (VERBOSE (tgs->handle))
     printf ("Building TGS-REQ...\n");
 
-  res =
-    shishi_apreq_options (tgs->handle, shishi_ap_req (tgs->ap), &apoptions);
+  res = shishi_kdcreq_build (tgs->handle, tgs->tgsreq);
+  if (res != SHISHI_OK)
+    return res;
+
+  res = shishi_apreq_options (tgs->handle, shishi_ap_req (tgs->ap),
+			      &apoptions);
   if (res != SHISHI_OK)
     {
       shishi_error_printf (tgs->handle,
