@@ -773,18 +773,88 @@ shishi_tkts_get (Shishi_tkts * tkts, Shishi_tkts_hint * hint)
   if (tgt == NULL)
     {
       Shishi_as *as;
+      time_t starttime = hint->starttime ? hint->starttime : time(NULL);
+      time_t endtime = hint->endtime ? hint->endtime :
+	starttime + tkts->handle->ticketlife;
+      time_t renew_till = hint->renew_till ? hint->renew_till :
+	starttime + tkts->handle->renewlife;
 
       /* Get TGT ... XXX cross realm */
 
       rc = shishi_as (tkts->handle, &as);
+      if (rc != SHISHI_OK)
+	{
+	  shishi_error_printf (tkts->handle, "Cannot create AS: %s",
+			       shishi_strerror (rc));
+	  return NULL;
+	}
+
+      if (hint->starttime)
+	{
+	  rc = shishi_asn1_write (tkts->handle, shishi_as_req(as),
+				  "req-body.from",
+				  shishi_generalize_time (tkts->handle,
+							  starttime),
+				  0);
+	  if (rc != SHISHI_OK)
+	    {
+	      shishi_error_printf (tkts->handle, "Cannot set starttime: %s",
+				   shishi_strerror (rc));
+	      return NULL;
+	    }
+	}
+
+      if (hint->endtime)
+	{
+	  rc = shishi_asn1_write (tkts->handle, shishi_as_req(as),
+				  "req-body.till",
+				  shishi_generalize_time (tkts->handle,
+							  endtime),
+				  0);
+	  if (rc != SHISHI_OK)
+	    {
+	      shishi_error_printf (tkts->handle, "Cannot set starttime: %s",
+				   shishi_strerror (rc));
+	      return NULL;
+	    }
+	}
+
+      if (hint->renewable)
+	{
+	  rc = shishi_kdcreq_options_add (tkts->handle, shishi_as_req(as),
+					  SHISHI_KDCOPTIONS_RENEWABLE);
+	  if (rc != SHISHI_OK)
+	    {
+	      shishi_error_printf (tkts->handle, "Cannot set KDC Options: %s",
+				   shishi_strerror (rc));
+	      return NULL;
+	    }
+
+	  rc = shishi_asn1_write (tkts->handle, shishi_as_req(as),
+				  "req-body.rtime",
+				  shishi_generalize_time
+				  (tkts->handle, renew_till),
+				  0);
+	  if (rc != SHISHI_OK)
+	    {
+	      shishi_error_printf (tkts->handle, "Cannot set KDC Options: %s",
+				   shishi_strerror (rc));
+	      return NULL;
+	    }
+	}
+
+      if (rc == SHISHI_OK)
+	rc = shishi_as_req_build (as);
       if (rc == SHISHI_OK)
 	rc = shishi_as_sendrecv (as);
       if (rc == SHISHI_OK)
 	rc = shishi_as_rep_process (as, NULL, hint->passwd);
       if (rc != SHISHI_OK)
 	{
-	  printf ("AS exchange failed: %s\n%s\n", shishi_strerror (rc),
-		  shishi_strerror_details (tkts->handle));
+	  shishi_error_printf (tkts->handle,
+			       "AS exchange failed: %s\n%s\n",
+			       shishi_strerror (rc),
+			       shishi_strerror_details (tkts->handle));
 	  if (rc == SHISHI_GOT_KRBERROR)
 	    shishi_krberror_pretty_print (tkts->handle, stdout,
 					  shishi_as_krberror (as));
@@ -795,8 +865,7 @@ shishi_tkts_get (Shishi_tkts * tkts, Shishi_tkts_hint * hint)
 
       if (!tgt)
 	{
-	  printf ("No ticket in AS-REP?!: %s\n",
-		  shishi_strerror_details (tkts->handle));
+	  shishi_error_printf (tkts->handle, "No ticket in AS-REP");
 	  return NULL;
 	}
 
@@ -902,24 +971,18 @@ shishi_tkts_get_for_localservicepasswd (Shishi_tkts * tkts,
 					const char *service,
 					const char *passwd)
 {
+  Shishi_tkt *tkt;
   Shishi_tkts_hint hint;
-  char buf[HOST_NAME_MAX];
   int ret;
 
-  strcpy (buf, service);
-  strcat (buf, "/");
-
-  ret = gethostname (&buf[strlen (service) + 1],
-		     sizeof (buf) - strlen (service) - 1);
-  buf[sizeof (buf) - 1] = '\0';
-
-  if (ret != 0)
-    strcpy (&buf[strlen (service) + 1], "localhost");
-
   memset (&hint, 0, sizeof (hint));
   hint.client = (char *) shishi_principal_default (tkts->handle);
-  hint.server = buf;
+  hint.server = shishi_server_for_local_service (tkts->handle, service);
   hint.passwd = (char *) passwd;
 
-  return shishi_tkts_get (tkts, &hint);
+  tkt = shishi_tkts_get (tkts, &hint);
+
+  free (hint.server);
+
+  return tkt;
 }
